#!/usr/bin/env python

"""
Licensed to the Apache Software Foundation (ASF) under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The ASF licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import sys
if sys.version_info[0] == 2:
    from urllib2 import urlopen
elif sys.version_info[0] == 3:
    from urllib.request import urlopen
import ambari_simplejson as json # simplejson is much faster comparing to Python 2.6 json module and has the same functions set.
import logging
import traceback

from resource_management.libraries.functions.curl_krb_request import curl_krb_request
from resource_management.libraries.functions.curl_krb_request import DEFAULT_KERBEROS_KINIT_TIMER_MS
from resource_management.libraries.functions.curl_krb_request import KERBEROS_KINIT_TIMER_PARAMETER
from resource_management.libraries.functions.curl_krb_request import CONNECTION_TIMEOUT_DEFAULT
from resource_management.core.environment import Environment

RECON_HTTP_ADDRESS_KEY = '{{ozone-site/ozone.recon.http-address}}'
RECON_HTTPS_ADDRESS_KEY = '{{ozone-site/ozone.recon.https-address}}'
RECON_HTTP_POLICY_KEY = '{{ozone-site/ozone.http.policy}}'

OZONE_SITE_KEY = '{{ozone-site}}'
KERBEROS_KEYTAB = '{{ozone-site/ozone.recon.http.auth.kerberos.keytab}}'
KERBEROS_PRINCIPAL = '{{ozone-site/ozone.recon.http.auth.kerberos.principal}}'
SECURITY_ENABLED_KEY = '{{cluster-env/security_enabled}}'
SMOKEUSER_KEY = "{{cluster-env/smokeuser}}"
EXECUTABLE_SEARCH_PATHS = '{{kerberos-env/executable_search_paths}}'
logger = logging.getLogger('ambari_alerts')

RESULT_STATE_CRITICAL = 'CRITICAL'
RESULT_STATE_WARNING = 'WARNING'
RESULT_STATE_UNKNOWN = 'UNKNOWN'
RESULT_STATE_SKIPPED = 'SKIPPED'
RESULT_STATE_OK = 'OK'

KEY_CAPACITY_USED_CRITICAL = 'capacity.used.critical.threshold'
KEY_CAPACITY_USED_WARNING = 'capacity.used.warning.threshold'
KEY_DATANODES_DEAD_CRITICAL = 'health.datanode.warning.critical'
KEY_DATANODES_DEAD_WARNING = 'health.datanode.warning'

def get_tokens():
  """
  Returns a tuple of tokens in the format {{site/property}} that will be used
  to build the dictionary passed into execute
  """
  return (OZONE_SITE_KEY, RECON_HTTP_ADDRESS_KEY, EXECUTABLE_SEARCH_PATHS,
          RECON_HTTPS_ADDRESS_KEY, RECON_HTTP_POLICY_KEY, SMOKEUSER_KEY, KERBEROS_KEYTAB, KERBEROS_PRINCIPAL, SECURITY_ENABLED_KEY)

def execute(configurations={}, parameters={}, host_name=None):
  """
  Returns a tuple containing the result code and a pre-formatted result label

  Keyword arguments:
  configurations : a mapping of configuration key to value
  parameters : a mapping of script parameter key to value
  host_name : the name of this host where the alert is running

  :type configurations dict
  :type parameters dict
  :type host_name str
  """

  if configurations is None:
    return (RESULT_STATE_UNKNOWN, ['There were no configurations supplied to the script.'])

  uri = None
  http_policy = 'HTTP_ONLY'

  # ozone-site is required
  if not OZONE_SITE_KEY in configurations:
    return (RESULT_STATE_UNKNOWN, ['{0} is a required parameter for the script'.format(OZONE_SITE_KEY)])

  if RECON_HTTP_POLICY_KEY in configurations:
    http_policy = configurations[RECON_HTTP_POLICY_KEY]

  if SMOKEUSER_KEY in configurations:
    smokeuser = configurations[SMOKEUSER_KEY]

  executable_paths = None
  if EXECUTABLE_SEARCH_PATHS in configurations:
    executable_paths = configurations[EXECUTABLE_SEARCH_PATHS]

  security_enabled = False
  if SECURITY_ENABLED_KEY in configurations:
    security_enabled = str(configurations[SECURITY_ENABLED_KEY]).upper() == 'TRUE'

  kerberos_keytab = None
  if KERBEROS_KEYTAB in configurations:
    kerberos_keytab = configurations[KERBEROS_KEYTAB]

  kerberos_principal = None
  if KERBEROS_PRINCIPAL in configurations:
    kerberos_principal = configurations[KERBEROS_PRINCIPAL]
    kerberos_principal = kerberos_principal.replace('_HOST', host_name)

  kinit_timer_ms = parameters.get(KERBEROS_KINIT_TIMER_PARAMETER, DEFAULT_KERBEROS_KINIT_TIMER_MS)

  # determine the right URI and whether to use SSL
  ozone_site = configurations[OZONE_SITE_KEY]

  scheme = "https" if http_policy == "HTTPS_ONLY" else "http"
  address = configurations[RECON_HTTPS_ADDRESS_KEY] if http_policy == "HTTPS_ONLY" else configurations[RECON_HTTP_ADDRESS_KEY]
  uri = "{0}:{1}".format(host_name, address.split(':')[1])
  ozone_cluster_state_uri = "{0}://{1}/api/v1/clusterState".format(scheme, uri)
  label = None
  result_code = "OK"

  try:
    if kerberos_principal is not None and kerberos_keytab is not None and security_enabled:
      env = Environment.get_instance()

      clusterStateResponse, error_msg, time_millis = curl_krb_request(
        env.tmp_dir, kerberos_keytab,
        kerberos_principal, ozone_cluster_state_uri, "ozone_cluster_state_uri", executable_paths, False,
        "Ozone Cluster State", smokeuser, kinit_timer_ms = kinit_timer_ms
       )

      cluster_state_response_json = json.loads(clusterStateResponse)
      if KEY_CAPACITY_USED_CRITICAL in parameters:
        (result_code, [label]) = generate_capacity_alerts_from_state(cluster_state_response_json, parameters)
      else:
        (result_code, [label]) = generate_datanode_alerts_from_state(cluster_state_response_json, parameters)



    else:
      cluster_state_response_json = get_value_from_api(ozone_cluster_state_uri)
      if KEY_CAPACITY_USED_CRITICAL in parameters:
        (result_code, [label]) = generate_capacity_alerts_from_state(cluster_state_response_json, parameters)
      else:
        (result_code, [label]) = generate_datanode_alerts_from_state(cluster_state_response_json, parameters)

  except:
    label = traceback.format_exc()
    result_code = 'UNKNOWN'

  return ((result_code, [label]))


def formatWithUnit(value_in_bytes):
  # Use float division to ensure compatibility between Python 2 and 3
  value_in_kb = float(value_in_bytes) / 1024
  value_in_mb = value_in_kb / 1024
  value_in_gb = value_in_mb / 1024
  value_in_tb = value_in_gb / 1024

  if value_in_tb >= 1:
    return "{} TB".format(round(value_in_tb, 1))
  elif value_in_gb >= 1:
    return "{} GB".format(round(value_in_gb, 1))
  elif value_in_mb >= 1:
    return "{} MB".format(round(value_in_mb, 1))
  else:
    return "{} KB".format(round(value_in_kb, 1))

def generate_datanode_alerts_from_state(state, params):
  # start out assuming an OK status
  label = None
  result_code = "OK"
  numDataNode = state['totalDatanodes']
  numHealthyDatanodes = state['healthyDatanodes']
  dead = numDataNode-numHealthyDatanodes

  message = "All {} DataNode(s) are healthy".format(numDataNode)
  if dead > params[KEY_DATANODES_DEAD_CRITICAL]:
    message = "DataNode Health: [Live={}, Stale={}]".format(numHealthyDatanodes, dead)
    return (RESULT_STATE_CRITICAL,[message])
  elif dead > params[KEY_DATANODES_DEAD_WARNING]:
    message = "DataNode Health: [Live={}, Stale={}]".format(numHealthyDatanodes, dead)
    return (RESULT_STATE_WARNING,[message])
  else:
    return (RESULT_STATE_OK,[message])

def generate_capacity_alerts_from_state(state, params):
  # start out assuming an OK status
  label = None
  result_code = "OK"
  
  ozone_total_capacity = state['storageReport']['capacity'] #by default in Bytes
  ozone_used_capacity = state['storageReport']['used'] #by default in Bytes
  ozone_remaining_capacity = state['storageReport']['remaining'] # by default in Bytes
  percent_used = float(ozone_used_capacity/ozone_total_capacity) * 100
  
  message = "Total Capacity:[{0:.0f}%, {1}], Capacity Remaining:[{2}]".format(percent_used, formatWithUnit(ozone_used_capacity), formatWithUnit(ozone_remaining_capacity))
  
  if percent_used >= params[KEY_CAPACITY_USED_CRITICAL]:
    return (RESULT_STATE_CRITICAL,[message])
  elif percent_used >= params[KEY_CAPACITY_USED_WARNING]:
    return (RESULT_STATE_WARNING,[message])
  else:
    return (RESULT_STATE_OK,[message])

def get_value_from_api(uri):
  """
   Read cluster state from recon ui

  :param uri: uri path
  :return: cluster state value
  
  :type query str
  :type cluster state json
  """
  response = None

  try:
    response = urlopen(uri, timeout=int(CONNECTION_TIMEOUT_DEFAULT))
    data = response.read()

    return json.loads(data)
  finally:
    if response is not None:
      try:
        response.close()
      except:
        pass
